"use strict";
jest.disableAutomock();
var edit_1 = require('prosemirror/dist/edit');
var schema_basic_1 = require('prosemirror/dist/schema-basic');
var index_1 = require('../index');
var body = document.querySelector('body');
var proseMirrorInstance;
function assertSchemaConvertion(input, expectedOutput) {
    proseMirrorInstance.input.insertText(proseMirrorInstance.selection.from, proseMirrorInstance.selection.to, input);
    expect(proseMirrorInstance.doc.toString()).toBe(expectedOutput);
}
function assertDOMConvertion(input, expectedOutput) {
    proseMirrorInstance.input.insertText(proseMirrorInstance.selection.from, proseMirrorInstance.selection.to, input);
    var transformedInput = proseMirrorInstance.doc.content.toDOM();
    var container = document.createElement('div');
    container.appendChild(transformedInput.cloneNode(true));
    expect(container.innerHTML).toBe(expectedOutput);
}
beforeEach(function () {
    proseMirrorInstance = new edit_1.ProseMirror({
        place: body.appendChild(document.createElement('div')),
        schema: schema_basic_1.schema,
        plugins: [
            new edit_1.Plugin(index_1["default"])
        ]
    });
});
afterEach(function () {
    proseMirrorInstance = null;
    body.innerHTML = '';
});
describe('MarkdownInputRulesPlugin - schema convertion', function () {
    it('should convert "**bold** to bold', function () {
        assertSchemaConvertion('**bold**', 'doc(paragraph(strong("bold")))');
    });
    it('should convert "__bold__ to bold', function () {
        assertSchemaConvertion('__bold__', 'doc(paragraph(strong("bold")))');
    });
    it('should convert "*italic*" to italic', function () {
        assertSchemaConvertion('*italic*', 'doc(paragraph(em("italic")))');
    });
    it('should convert "_italic_" to italic', function () {
        assertSchemaConvertion('_italic_', 'doc(paragraph(em("italic")))');
    });
    it('should convert "[text](link)" to hyperlink', function () {
        assertSchemaConvertion('[http://atlassian.com](http://atlassian.com)', 'doc(paragraph(link("http://atlassian.com")))');
    });
    it('should convert "***" to horizontal rule', function () {
        assertSchemaConvertion('***', 'doc(paragraph, horizontal_rule, paragraph)');
    });
    it('should convert "---" to horizontal rule', function () {
        assertSchemaConvertion('---', 'doc(paragraph, horizontal_rule, paragraph)');
    });
    it('should convert `code` to inline code', function () {
        assertSchemaConvertion('`code`', 'doc(paragraph(code("code")))');
    });
    it('should convert `![Image](URL)` to image', function () {
        assertSchemaConvertion('![Image](http://url/a.png)', 'doc(paragraph(image))');
    });
});
describe('MarkdownInputRulesPlugin - markup convertion', function () {
    it('should convert "**bold** to bold', function () {
        assertDOMConvertion('**bold**', '<p><strong>bold</strong></p>');
    });
    it('should convert "__bold__ to bold', function () {
        assertDOMConvertion('__bold__', '<p><strong>bold</strong></p>');
    });
    it('should convert "*italic*" to italic', function () {
        assertDOMConvertion('*italic*', '<p><em>italic</em></p>');
    });
    it('should convert "_italic_" to italic', function () {
        assertDOMConvertion('_italic_', '<p><em>italic</em></p>');
    });
    it('should convert to hyperlink', function () {
        assertDOMConvertion('[http://atlassian.com](http://atlassian.com)', '<p><a href="http://atlassian.com">http://atlassian.com</a></p>');
    });
    it('should convert "***" to horizontal rule', function () {
        assertDOMConvertion('***', '<p></p><div><hr></div><p></p>');
    });
    it('should convert "---" to horizontal rule', function () {
        assertDOMConvertion('---', '<p></p><div><hr></div><p></p>');
    });
    it('should convert to inline code', function () {
        assertDOMConvertion('`code`', '<p><code>code</code></p>');
    });
    it('should convert to image', function () {
        assertDOMConvertion('![Image](http://url/a.png)', '<p><img src="http://url/a.png" alt="Image" title="Image"></p>');
    });
});
describe('MarkdownInputRulesPlugin - interactions', function () {
    it('should remove active marks after transformed output from schema', function () {
        assertSchemaConvertion('*italic*', 'doc(paragraph(em("italic")))');
        assertSchemaConvertion('m', 'doc(paragraph(em("italic"), "m"))');
    });
    it('should remove active marks after transformed output from markup', function () {
        assertDOMConvertion('*italic*', '<p><em>italic</em></p>');
        assertDOMConvertion('m', '<p><em>italic</em>m</p>');
    });
});
