"use strict";
var inputrules_1 = require('prosemirror/dist/inputrules');
function replaceWithNode(pm, match, pos, node) {
    var start = pos - match[0].length;
    var end = pos;
    pm.tr.delete(start, end).insert(start, node).apply();
    return true;
}
function replaceWithMark(pm, match, pos, mark) {
    var schema = pm.schema;
    var to = pos;
    var from = pos - match[1].length;
    var markType = schema.mark(mark);
    pm.tr.replaceWith(from, to, schema.text(match[2], markType)).apply();
    pm.removeActiveMark(markType);
    return true;
}
// [something](link) should convert to a hyperlink
var linkRule = new inputrules_1.InputRule(/\[(\S+)\]\((\S+)\)$/, ')', function (pm, match, pos) {
    var schema = pm.schema;
    var to = pos;
    var from = pos - match[0].length;
    var markType = schema.mark('link', {
        href: match[2]
    });
    pm.tr.replaceWith(from, to, schema.text(match[1], markType)).apply();
    pm.removeActiveMark(markType);
    return true;
});
// ![Image](http://url/a.png) should add an image
// Note: You have to load this rule before the link rule.
var imgRule = new inputrules_1.InputRule(/!\[(\S+)\]\((\S+)\)$/, ')', function (pm, match, pos) {
    var attrs = {
        src: match[2],
        alt: match[1],
        title: match[1]
    };
    var node = pm.schema.nodes.image.create(attrs);
    return replaceWithNode(pm, match, pos, node);
});
// **string** should bold the text
var strongRule1 = new inputrules_1.InputRule(/(\*\*([^\*]+)\*\*)$/, '*', function (pm, match, pos) {
    return replaceWithMark(pm, match, pos, 'strong');
});
// __string__ should bold the text
var strongRule2 = new inputrules_1.InputRule(/(__([^_]+)__)$/, '_', function (pm, match, pos) {
    return replaceWithMark(pm, match, pos, 'strong');
});
// _string_ or *string* should change the text to italic
var emRule1 = new inputrules_1.InputRule(/(?:[^\*]+)(\*([^\*]+?)\*)$|^(\*([^\*]+)\*)$/, '*', function (pm, match, pos) {
    return replaceWithMark(pm, match.filter(function (m) { return m !== undefined; }), pos, 'em');
});
var emRule2 = new inputrules_1.InputRule(/(?:[^_]+)(_([^_]+?)_)$|^(_([^_]+)_)$/, '_', function (pm, match, pos) {
    return replaceWithMark(pm, match.filter(function (m) { return m !== undefined; }), pos, 'em');
});
// `string` should change the current text to inline code block
var inlineCodeRule = new inputrules_1.InputRule(/(`([^`]+)`)$/, '`', function (pm, match, pos) {
    return replaceWithMark(pm, match, pos, 'code');
});
// --- or *** should add a horizontal line
var hrRule1 = new inputrules_1.InputRule(/\*\*\*$/, '*', function (pm, match, pos) {
    return replaceWithNode(pm, match, pos, pm.schema.nodes.horizontal_rule.create());
});
var hrRule2 = new inputrules_1.InputRule(/\-\-\-$/, '-', function (pm, match, pos) {
    return replaceWithNode(pm, match, pos, pm.schema.nodes.horizontal_rule.create());
});
var MarkdownInputRulesPlugin = (function () {
    function MarkdownInputRulesPlugin(editorInstance) {
        var pm = editorInstance;
        this.name = 'MarkdownInputRulesPlugin';
        this.inputRules = [
            strongRule1,
            strongRule2,
            emRule1,
            emRule2,
            inlineCodeRule,
            imgRule,
            linkRule,
            hrRule1,
            hrRule2,
        ].concat(inputrules_1.allInputRules);
        var rules = inputrules_1.inputRules.ensure(pm);
        this.inputRules.forEach(function (rule) { return rules.addRule(rule); });
    }
    MarkdownInputRulesPlugin.prototype.detach = function (editorInstance) {
        var pm = editorInstance;
        var rules = inputrules_1.inputRules.ensure(pm);
        this.inputRules.forEach(function (rule) { return rules.removeRule(rule); });
    };
    return MarkdownInputRulesPlugin;
}());
exports.__esModule = true;
exports["default"] = MarkdownInputRulesPlugin;
