jest.disableAutomock();

import {
  Plugin,
  ProseMirror,
} from 'prosemirror/dist/edit';

import {
  schema
} from 'prosemirror/dist/schema-basic';

import MarkdownInputRulesPlugin from '../index';

const body = document.querySelector('body');

let proseMirrorInstance: ProseMirror;

function assertSchemaConvertion(
  input: string,
  expectedOutput: string
) : void {
  proseMirrorInstance.input.insertText(
    proseMirrorInstance.selection.from,
    proseMirrorInstance.selection.to,
    input
  );

  expect(proseMirrorInstance.doc.toString()).toBe(expectedOutput);
}

function assertDOMConvertion(
  input: string,
  expectedOutput: string
) : void {
  proseMirrorInstance.input.insertText(
    proseMirrorInstance.selection.from,
    proseMirrorInstance.selection.to,
    input
  );

  const transformedInput = proseMirrorInstance.doc.content.toDOM();

  const container = document.createElement('div');

  container.appendChild(transformedInput.cloneNode(true));

  expect(container.innerHTML).toBe(expectedOutput);
}

beforeEach(() => {
  proseMirrorInstance = new ProseMirror(
    {
      place: body.appendChild(
        document.createElement('div')
      ),
      schema: schema,
      plugins: [
        new Plugin(MarkdownInputRulesPlugin)
      ]
    }
  );
});

afterEach(() => {
  proseMirrorInstance = null;
  body.innerHTML = '';
});

describe('MarkdownInputRulesPlugin - schema convertion', () => {
  it('should convert "**bold** to bold', () => {
    assertSchemaConvertion(
      '**bold**',
      'doc(paragraph(strong("bold")))'
    );
  });

  it('should convert "__bold__ to bold', () => {
    assertSchemaConvertion(
      '__bold__',
      'doc(paragraph(strong("bold")))'
    );
  });

  it('should convert "*italic*" to italic', () => {
    assertSchemaConvertion(
      '*italic*',
      'doc(paragraph(em("italic")))'
    );
  });

  it('should convert "_italic_" to italic', () => {
    assertSchemaConvertion(
      '_italic_',
      'doc(paragraph(em("italic")))'
    );
  });

  it('should convert "[text](link)" to hyperlink', () => {
    assertSchemaConvertion(
      '[http://atlassian.com](http://atlassian.com)',
      'doc(paragraph(link("http://atlassian.com")))'
    );
  });

  it('should convert "***" to horizontal rule', () => {
    assertSchemaConvertion(
      '***',
      'doc(paragraph, horizontal_rule, paragraph)'
    );
  });

  it('should convert "---" to horizontal rule', () => {
    assertSchemaConvertion(
      '---',
      'doc(paragraph, horizontal_rule, paragraph)'
    );
  });

  it('should convert `code` to inline code', () => {
    assertSchemaConvertion(
      '`code`',
      'doc(paragraph(code("code")))'
    );
  });

  it('should convert `![Image](URL)` to image', () => {
    assertSchemaConvertion(
      '![Image](http://url/a.png)',
      'doc(paragraph(image))'
    );
  });
});

describe('MarkdownInputRulesPlugin - markup convertion', () => {
  it('should convert "**bold** to bold', () => {
    assertDOMConvertion(
      '**bold**',
      '<p><strong>bold</strong></p>'
    );
  });

  it('should convert "__bold__ to bold', () => {
    assertDOMConvertion(
      '__bold__',
      '<p><strong>bold</strong></p>'
    );
  });

  it('should convert "*italic*" to italic', () => {
    assertDOMConvertion(
      '*italic*',
      '<p><em>italic</em></p>'
    );
  });

  it('should convert "_italic_" to italic', () => {
    assertDOMConvertion(
      '_italic_',
      '<p><em>italic</em></p>'
    );
  });

  it('should convert to hyperlink', () => {
    assertDOMConvertion(
      '[http://atlassian.com](http://atlassian.com)',
      '<p><a href="http://atlassian.com">http://atlassian.com</a></p>'
    );
  });

  it('should convert "***" to horizontal rule', () => {
    assertDOMConvertion(
      '***',
      '<p></p><div><hr></div><p></p>'
    );
  });

  it('should convert "---" to horizontal rule', () => {
    assertDOMConvertion(
      '---',
      '<p></p><div><hr></div><p></p>'
    );
  });

  it('should convert to inline code', () => {
    assertDOMConvertion(
      '`code`',
      '<p><code>code</code></p>'
    );
  });

  it('should convert to image', () => {
    assertDOMConvertion(
      '![Image](http://url/a.png)',
      '<p><img src="http://url/a.png" alt="Image" title="Image"></p>'
    );
  });
});

describe('MarkdownInputRulesPlugin - interactions', () => {
  it('should remove active marks after transformed output from schema', () => {
    assertSchemaConvertion(
      '*italic*',
      'doc(paragraph(em("italic")))'
    );

    assertSchemaConvertion(
      'm',
      'doc(paragraph(em("italic"), "m"))'
    );
  });

  it('should remove active marks after transformed output from markup', () => {
    assertDOMConvertion(
      '*italic*',
      '<p><em>italic</em></p>'
    );

    assertDOMConvertion(
      'm',
      '<p><em>italic</em>m</p>'
    );
  });
});
