import { InputRule, inputRules, allInputRules } from 'prosemirror/dist/inputrules';
function replaceWithNode(pm, match, pos, node) {
    const start = pos - match[0].length;
    const end = pos;
    pm.tr.delete(start, end).insert(start, node).apply();
    return true;
}
function replaceWithMark(pm, match, pos, mark) {
    const schema = pm.schema;
    const to = pos;
    const from = pos - match[1].length;
    const markType = schema.mark(mark);
    pm.tr.replaceWith(from, to, schema.text(match[2], markType)).apply();
    pm.removeActiveMark(markType);
    return true;
}
// [something](link) should convert to a hyperlink
const linkRule = new InputRule(/\[(\S+)\]\((\S+)\)$/, ')', (pm, match, pos) => {
    const schema = pm.schema;
    const to = pos;
    const from = pos - match[0].length;
    const markType = schema.mark('link', {
        href: match[2],
    });
    pm.tr.replaceWith(from, to, schema.text(match[1], markType)).apply();
    pm.removeActiveMark(markType);
    return true;
});
// ![Image](http://url/a.png) should add an image
// Note: You have to load this rule before the link rule.
const imgRule = new InputRule(/!\[(\S+)\]\((\S+)\)$/, ')', (pm, match, pos) => {
    const attrs = {
        src: match[2],
        alt: match[1],
        title: match[1],
    };
    const node = pm.schema.nodes.image.create(attrs);
    return replaceWithNode(pm, match, pos, node);
});
// **string** should bold the text
const strongRule1 = new InputRule(/(\*\*([^\*]+)\*\*)$/, '*', (pm, match, pos) => replaceWithMark(pm, match, pos, 'strong'));
// __string__ should bold the text
const strongRule2 = new InputRule(/(__([^_]+)__)$/, '_', (pm, match, pos) => replaceWithMark(pm, match, pos, 'strong'));
// _string_ or *string* should change the text to italic
const emRule1 = new InputRule(/(?:[^\*]+)(\*([^\*]+?)\*)$|^(\*([^\*]+)\*)$/, '*', (pm, match, pos) => replaceWithMark(pm, match.filter((m) => m !== undefined), pos, 'em'));
const emRule2 = new InputRule(/(?:[^_]+)(_([^_]+?)_)$|^(_([^_]+)_)$/, '_', (pm, match, pos) => replaceWithMark(pm, match.filter((m) => m !== undefined), pos, 'em'));
// `string` should change the current text to inline code block
const inlineCodeRule = new InputRule(/(`([^`]+)`)$/, '`', (pm, match, pos) => replaceWithMark(pm, match, pos, 'code'));
// --- or *** should add a horizontal line
const hrRule1 = new InputRule(/\*\*\*$/, '*', (pm, match, pos) => replaceWithNode(pm, match, pos, pm.schema.nodes.horizontal_rule.create()));
const hrRule2 = new InputRule(/\-\-\-$/, '-', (pm, match, pos) => replaceWithNode(pm, match, pos, pm.schema.nodes.horizontal_rule.create()));
class MarkdownInputRulesPlugin {
    constructor(editorInstance) {
        const pm = editorInstance;
        this.name = 'MarkdownInputRulesPlugin';
        this.inputRules = [
            strongRule1,
            strongRule2,
            emRule1,
            emRule2,
            inlineCodeRule,
            imgRule,
            linkRule,
            hrRule1,
            hrRule2,
        ].concat(allInputRules);
        const rules = inputRules.ensure(pm);
        this.inputRules.forEach((rule) => rules.addRule(rule));
    }
    detach(editorInstance) {
        const pm = editorInstance;
        const rules = inputRules.ensure(pm);
        this.inputRules.forEach((rule) => rules.removeRule(rule));
    }
}
export default MarkdownInputRulesPlugin;
